# MineSweeperModule: Contains the classes for the game's GUI
module MineSweeperGUIModule

begin
  require 'rubygems' 
rescue LoadError
end
require 'wx'

require 'MineSweeperConnectionModule'
require 'MineSweeperUtilModule'
# LoginFrame:
#  This is the frame for select nickname
class LoginFrame < Wx::Frame
  # @return [MineSweeperApplication] The owner of the frame
  attr_reader :owner
  # Wx id for the logon frame
  WxLOGON = 1
  # Wx id for the nick box
  WxLOGONNICK = 2
  # Wx id for the exit button
  WxLOGONEXIT = 3
  # Wx id for the login button
  WxLOGONLOGON = 4
  
  # Initialize the form
  # @param [String] title Frame's title
  # @param [MineSweeperApplication] owner The Frame's owner
  def initialize(title,owner)
    @owner = owner
    # The main application frame has no parent (nil)
    super(nil, :title => title, :size => [ 330, 140 ], :id => WxLOGON)
    @panel = Wx::Panel.new(self, -1)
    @nicklabel =  Wx::StaticText.new(@panel, :pos => Wx::Point.new(15,20), :label => "Nicknev")
    @nickbox   =  Wx::TextCtrl.new(@panel, :size => Wx::Size.new(300,20), :pos => Wx::Point.new(15,40), :id => WxLOGONNICK, :style=>Wx::TE_PROCESS_ENTER)
    @exitbutton = Wx::Button.new(@panel, :size => Wx::Size.new(80,20), :pos => Wx::Point.new(60,70), :label=>"Kilepes", :id => WxLOGONEXIT)
    @loginbutton = Wx::Button.new(@panel, :size => Wx::Size.new(105,20), :pos => Wx::Point.new(170,70), :label=>"Bejelentkezes", :id => WxLOGONLOGON)
    @panel.evt_button(@exitbutton.get_id) {  @owner.on_quit unless @owner == nil }
    @panel.evt_button(@loginbutton.get_id) { @owner.startLogin(@nickbox.get_value) unless @owner == nil }
    @panel.evt_text_enter(@nickbox.get_id) { @owner.startLogin(@nickbox.get_value) unless @owner == nil }
    evt_close { @owner.on_quit unless @owner == nil }
    centre_on_parent()
    set_theme_enabled(true)
  end

  # Clear the nick box
  # @return [NotImportant] Not important value
  def clear
    @nickbox.set_value("")
  end
end

#  This contains playerlist, show chat messages and allow to select opponent
class MainFrame < Wx::Frame

  # @return [MineSweeperApplication] The owner of the frame
  attr_reader :owner
  # @return [String] Nickname
  attr_accessor :nick
  # Wx id for the main frame
  WxMAIN = 5
  # Wx id for the player list
  WxMAINLIST = 6
  # Wx id for the chat box
  WxMAINCHAT = 7
  # Wx id for the editpr box
  WxMAINBOX = 8
  # Wx id for the send button
  WxMAINSEND = 9
  # Wx id for the logout button
  WxMAINLOGOUT = 10
  # Wx id for the exit button
  WxMAINEXIT = 11
  # Wx id for the game request button
  WxMAINGAME = 12
  
  # Initialize the form
  # @param [String] title Frame's title
  # @param [MineSweeperApplication] owner The Frame's owner
  def initialize(title,owner)
    @owner = owner
    @nick = nil;
    @selected = nil;
    # The main application frame has no parent (nil)
    super(nil, :title => title, :size => [ 718, 420 ], :id => WxMAIN)
    @panel = Wx::Panel.new(self, -1)
    @players =  Wx::ListBox.new(@panel, :pos => Wx::Point.new(10,40), :size => Wx::Size.new(251,331), :id=>WxMAINLIST)
    @panel.evt_listbox(@players.get_id) { | e | @selected = e.get_string }
    @playerslabel = Wx::StaticText.new(@panel, :pos => Wx::Point.new(12,15), :label => "Elerheto jatekosok")
    @chat =  Wx::ListBox.new(@panel, :pos => Wx::Point.new(270,71), :size => Wx::Size.new(421,271), :id=>WxMAINLIST)
    @chatlabel = Wx::StaticText.new(@panel, :pos => Wx::Point.new(270,50), :label => "Kozos chat")
    @exitbutton = Wx::Button.new(@panel, :size => Wx::Size.new(121,23), :pos => Wx::Point.new(280,10), :label=>"Kilepes", :id => WxMAINEXIT)
    @panel.evt_button(@exitbutton.get_id) { @players.clear; @chat.clear; @owner.on_quit unless @owner == nil }
    @logoutbutton = Wx::Button.new(@panel, :size => Wx::Size.new(121,23), :pos => Wx::Point.new(410,10), :label=>"Kijelentkezes", :id => WxMAINLOGOUT)
    @panel.evt_button(@logoutbutton.get_id) { @players.clear; @chat.clear; @owner.on_logout unless @owner == nil }
    @gamebutton = Wx::Button.new(@panel, :size => Wx::Size.new(121,23), :pos => Wx::Point.new(540,10), :label=>"Kihivas jatekra", :id => WxMAINGAME)
    @panel.evt_button(@gamebutton.get_id) { @owner.on_game_request(@selected) unless (@owner == nil or @selected.nil?) }
    @editor = Wx::TextCtrl.new(@panel, :size => Wx::Size.new(370,20), :pos => Wx::Point.new(270,342), :id => WxMAINBOX, :style=>Wx::TE_PROCESS_ENTER)
    @panel.evt_text_enter(@editor.get_id) { txt = @editor.get_value; @editor.set_value(""); if (!txt.eql?("")) then @owner.sendGlobMsg(txt) end  }
    @chatbutton = Wx::Button.new(@panel, :size => Wx::Size.new(50,20), :pos => Wx::Point.new(640,342), :label=>"Kuld", :id => WxMAINSEND)
    @panel.evt_button(@chatbutton.get_id) { txt = @editor.get_value; @editor.set_value(""); if (!txt.eql?("")) then @owner.sendGlobMsg(txt) end }
    evt_close { @players.clear; @chat.clear; @owner.on_logout unless @owner == nil}
    centre_on_parent()
    set_theme_enabled(true)
    #evt_idle {Thread.pass}
  end
  
  # Set the playerlist
  # @param [Array(String)] list The list of nicknames
  def setUserList(list)
    @players.insert_items(list,0)
  end
  
  # Shows the received global message
  # @param [String] sender Sender's nickname
  # @param [Array(String)] msg The received message
  def gotGlobMsg(sender,msg)
    @chat.append("#{sender} -> #{msg.join('#')}")
    @chat.refresh(true)
  end
 
  # Add the joined player's nick to the list
  # @param [String] nick The nickname
  def joined(nick)
    @players.delete(@players.find_string(nick))
    @players.append(nick)
  end
  # Remove the disconnected player's nick from the list
  # @param [String] nick The nickname 
  def left(nick)
    @players.delete(@players.find_string(nick))
  end
  
end

#  Simple panel that shows a minesweeper
#  {MineSweeperUtilModule::MineSweeperGame game}
class SweeperPanel < Wx::Panel
  # @return [MineSweeperUtilModule::MineSweeperGame] The game object
  attr_reader :game
  
  # @return [GameFrame] The owner of the frame
  attr_reader :owner
  
  # Initialize the panel
  # @param [MineSweeperUtilModule::MineSweeperGame] game The game object
  # @param [GameFrame] owner The owner of the panel
  # @return [NotImportant] Not important value
  def init(game,owner)
    @pics = []
    @game = game
    @owner = owner
    @w = 20
    @h = 20
    (0..11).each { |x| @pics <<  Wx::Bitmap.from_image(Wx::Image.new("pics/#{x}.png").scale(@w,@h)) }
    evt_paint :on_paint
    evt_left_down() { | e | @owner.hit((e.get_x() - 20)/@w,(e.get_y() - 20)/@h) }
  end
  
  # Paints the game to the panel
  # @return [NotImportant] Not important value
  def on_paint
    paint_buffered do | dc |
      pen = Wx::Pen.new(Wx::LIGHT_GREY)
      brush = Wx::Brush.new()
      brush.set_colour(Wx::LIGHT_GREY)

      dc.set_pen(pen)
      dc.set_brush(brush)
      dc.draw_rectangle(0,0,self.get_size.get_width,self.get_size.get_height)
      (0..(@game.size-1)).each do |x|
        (0..(@game.size-1)).each do |y|
          type = @game.get(x,y).visible ? @game.get(x,y).type : 9
          dc.draw_bitmap(@pics[type], 20 + @w*x, 20+@h*y, false)
        end
      end
    end
  end
 
end

# Class for show {MineSweeperUtilModule::MineSweeperGame game}'s details
class GameFrame < Wx::Frame
  # return [String] The opponent's nick
  attr_reader :opp
  # return [MineSweeperApplication] The owner of the frame
  attr_reader :owner
  # return [SweeperPanel] The panel with game
  attr_reader :panel
  # return [MineSweeperUtilModule::MineSweeperGame] The game object
  attr_reader :game
  # Wx id for game panel
  WxGAMEPANEL = 101
  # Wx id for the game frame
  WxGAME = 100
  # Initialize the frame
  # @param [String] title The frame's title
  # @param [Boolean] active True if we will start the game
  # @param [String] opp The opponent's nick
  # @param [String] table String representation of the game table
  # @param [MineSweeperApplication] owner The frame's owner
  def initialize(title,active,opp,table,owner)
    @owner = owner
    @opp = opp
    super(nil, :title => title, :size => [ 600, 400 ], :id => WxGAME)
    @panel = SweeperPanel.new(self, :pos => [40,40], :size => [330,330], :id=>WxGAMEPANEL)
    @game = MineSweeperUtilModule::MineSweeperGame.new
    @game.setTable(table,Math.sqrt(table.size).to_i)
    @game.start(active.to_i == 1)
    @panel.init(@game,self)
    @left = Wx::StaticText.new(@panel, :pos => Wx::Point.new(400,40), :label => "Fennmarado: #{@game.hiddenMines}");
    @scorelabel = Wx::StaticText.new(@panel, :pos => Wx::Point.new(400,60), :label => "Pontszamod: 0")
    @oppscorelabel = Wx::StaticText.new(@panel, :pos => Wx::Point.new(400,80), :label => "Ellenfeled pontszama: 0")
    @player = Wx::StaticText.new(@panel, :pos => Wx::Point.new(400,100), :label => @game.active ? "Te jossz" : "Ellenfeled jon")
    set_theme_enabled(true)
    evt_close { @owner.gameClosed(opp) unless @owner.nil?}
  end
  
  # Handler for a hit message
  # @param [Integer] x The x coordinate
  # @param [Integer] y The y coordinate
  # @return [NotImportant] Not important value
  def doHit(x,y);
    if (@game.state.eql?("ingame"))
      @game.hit(x,y);
      @scorelabel.set_label("Pontszamod: #{@game.score.to_s}");
      @oppscorelabel.set_label("Ellenfeled pontszama: #{@game.oppscore.to_s}");
      @left.set_label("Fennmarado: #{@game.hiddenMines}");
      @player.set_label(@game.active ? "Te jossz" : "Ellenfeled jon")
	  if (@game.state.eql?("loose")) then @owner.sendLost(opp); @player.set_label("Vesztettel") end
      refresh;
    end
  end
  # Call {MineSweeperApplication#hit the application hit method} that will send an appropriate hit message if
  # the game is active and the coordinates are valid
  # @param [Integer] x The x coordinate
  # @param [Integer] y The y coordinate
  # @return [NotImportant] Not important value
  def hit(x,y); if (@game.active and x >=0 and x<@game.size and y >=0 and y<@game.size) then @owner.hit(x,y,@opp) end end
end

# The client application 
class MineSweeperApplication < Wx::App

  # The {MineSweeperGUIModule::GameFrame game frames} of the current games
  attr_reader :gameFrames
  
  # Initialize the application
  # @return [NotImportant] Not important value
  def on_init
    set_use_best_visual(true)
    self.app_name = 'MineSweeper'
    @gameFrames = Hash.new
    @loginframe = LoginFrame.new("Bejelentkezes",self)
    @loginframe.show
    @mainframe = MainFrame.new("Kezdodhet a jatek:)!",self)
    @client = MineSweeperConnectionModule::MineSweeperClient.new(self)
    @client.start
    @timer = Wx::Timer.new(self,0)
    evt_timer(@timer.get_id) do
      Thread.pass
    end
    @timer.start(25)
  end
  
  
  # Shows the {MineSweeperGUIModule::MainFrame main frame} with the nick
  # @param [String] nick The nickname
  # @return [NotImportant] Not important value
  def doLogin(nick)
    @loginframe.clear
    @loginframe.hide
    @mainframe.nick = nick
    @mainframe.show
  end
  
  # Show a messagebox if the nick is exists on the {MineSweeperConnectionModule::MineSweeperServer server}
  # @return [NotImportant] Not important value
  def onNickExists
    @existsTimer = nil
    @existsTimer = Wx::Timer.new(self,3)
    evt_timer(@existsTimer.get_id) do
      Wx::message_box("A nev foglalt valassz masikat","Hiba",Wx::OK)
    end
    @existsTimer.start(100,true)
  end
  
  # Exit from the application
  # @return [NotImportant] Not important value
  def on_quit
    @client.send("exit#")
    @client.stop
	@mainframe.destroy
	@loginframe.destroy
    Thread.new { sleep 0.1; self.exit_main_loop}
  end
  
  # Perform a logout procedure and show {LoginFrame login} frame
  # @return [NotImportant] Not important value
  def on_logout
    @client.send("logout#")
    @mainframe.hide
    @loginframe.show
  end
  
  # Send an appropiate game request to the {MineSweeperConnectionModule::MineSweeperServer server} with 
  # {MineSweeperConnectionModule::MineSweeperClient client}.
  # @param [String] nick The nickname 
  # @return [NotImportant] Not important value
  def on_game_request(nick)
    if (!nick.nil? and !nick.eql?("")) then @client.send("request#game##{nick}") end
  end
  
  # Send a login message to the {MineSweeperConnectionModule::MineSweeperServer server} with 
  # {MineSweeperConnectionModule::MineSweeperClient client}. 
  # This method validate the nick.
  # @param [String] nick The required nickname
  # @return [NotImportant] Not important value
  def startLogin(nick)
    if (nick.match('^[0-9a-zA-Z]+$'))
      @client.send("login##{nick}")
    else
      Wx::MessageDialog.new(@loginframe,"A nicknev csak az angol abece kis es nagybetuit valamint szamot tartalmazhat es nem lehet ures!","Nick hiba").show_modal
    end
  end
  
  # Show a message box about the selected player no longer exists
  # @param [String] reason The reason of failure
  # @return [NotImportant] Not important value
  def doFailedRequest(reason)
    @failedTimer = nil
    @failedTimer = Wx::Timer.new(self,1)
    evt_timer(@failedTimer.get_id) do
	 Wx::message_box("A szemely mar nem elerheto","Sajnaljuk",Wx::OK)
    end
    @failedTimer.start(100,true)
  end
  
 
  # Show a pop up window about a player wants to play with you.
  # @param [String] opp The opponent's nickname
  # @return [NotImportant] Not important value
  def doGameRequest(opp)
    @requestTimer = nil
    @requestTimer = Wx::Timer.new(self,1)
    evt_timer(@requestTimer.get_id) do
      if (Wx::message_box("#{opp} akar veled jatszani. Elfogadod?","Meghivas",Wx::YES_NO) == Wx::YES)
        @client.send("game#accept##{opp}");
      else
        @client.send("game#reject##{opp}");
      end
    end
    @requestTimer.start(100,true)
  end
  
  # Show a pop up window about the opponent closed the game
  # @param [String] opp The opponent's nickname
  # @return [NotImportant] Not important value
  def doGameClosed(opp)
    @closedTimer = nil
    @closedTimer = Wx::Timer.new(self,2)
    evt_timer(@closedTimer.get_id) do
      Wx::message_box("#{opp} bezarta a jatekot","Sajnalom",Wx::OK)
      closeGameFrame(opp)
    end
    @closedTimer.start(100,true)
  end
  
  # Show a pop up window about the opponent don't want to play
  # @param [String] opp The opponent's nickname
  # @return [NotImportant] Not important value
  def doGameReject(opp)
    @rejectTimer = nil
    @rejectTimer = Wx::Timer.new(self,2)
    evt_timer(@rejectTimer.get_id) do
      Wx::message_box("#{opp} nem akar veled jatszani. Sajnalom:(","Sajnalom",Wx::OK)
    end
    @rejectTimer.start(100,true)
  end
  
  # Pop up a window about the {MineSweeperConnectionModule::MineSweeperServer server} disconnected, stops the 
  # {MineSweeperConnectionModule::MineSweeperClient client} and exitz from {MineSweeperApplication application}
  # @return [NotImportant] Not important value
  def notConnected
    @connectTimer = nil
    @connectTimer = Wx::Timer.new(self,5)
    evt_timer(@connectTimer.get_id) do
      Wx::message_box("A szerver szetkapcsolt a program kilep","Hiba",Wx::OK)
      @client.stop
      self.exit_main_loop
    end
    @connectTimer.start(100,true)
  end
  
  # Send a game closed message to the {MineSweeperConnectionModule::MineSweeperServer server} with 
  # {MineSweeperConnectionModule::MineSweeperClient client}.
  # @param [String] opp The opponent's nickname 
  # @return [NotImportant] Not important value
  def gameClosed(opp)
    @client.send("game#closed##{opp}")
    closeGameFrame(opp)
  end
  
  # Send global message to {MineSweeperConnectionModule::MineSweeperServer server} with 
  # {MineSweeperConnectionModule::MineSweeperClient client}.
  # @param [String] txt The message
  # @return [NotImportant] Not important value
  def sendGlobMsg(txt); @client.send("globmsg##{txt}") end
  
  # Handle received global message with {MainFrame#gotGlobMsg gotGlobMsg} method from {MainFrame MainFrame} class
  # @param [String] sender The sender's nickname
  # @param [Array(String)] msg The message 
  def gotGlobMsg(sender,msg); @mainframe.gotGlobMsg(sender,msg) end
 
  # Add a player nick to user list with {MainFrame#joined joined} method from {MainFrame MainFrame} class
  # @param [String] nick The nickname 
  # @return [NotImportant] Not important value
  def joined(nick); @mainframe.joined(nick) end
 
  # Delete nick from list with {MainFrame#left left} method from {MainFrame MainFrame} class, 
  # and close {GameFrame game frame} if we playing with that player with {#closeGameFrame closeGameFrame} method
  # @param [String] nick The nickname 
  # @return [NotImportant] Not important value
  def left(nick)
    @mainframe.left(nick)
    if (@gameFrames.has_key?(nick))
      Wx::message_box("#{nick} bezarta a jatekot","Sajnalom",Wx::OK)
      closeGameFrame(nick)
    end
  end
 
  # Set the player list with {MainFrame#setUserList setUserList} method from {MainFrame MainFrame} class
  # @param [Array(String)] list The nicknames
  # @return [NotImportant] Not important value
  def setUserList(list); @mainframe.setUserList(list) end
 
  # Creates and shows new {GameFrame GameFrame}
  # @param [String] opp The opponent's nickname
  # @param [Boolean] active True if we will start the game
  # @param [String] tablestring The string representation of the game table
  # @return [NotImportant] Not important value
  def startGame(opp,active,tablestring)
    @gameFrames[opp] = GameFrame.new("Sok szerencset #{@mainframe.nick}!",active,opp,tablestring,self)
    @gameFrames[opp].show
  end

  # Send an appropriate hit message
  # @param [Integer] x The x coordinate
  # @param [Integer] y The y coordinate
  # @param [String] opp The opponent's nickname
  # @return [NotImportant] Not important value
  def hit(x,y,opp)
    @client.send("hit##{opp}##{x}##{y}")
  end
 
  # Handle received hit message with {GameFrame#doHit doHit} method from {GameFrame GameFrame} class
  # @param [Integer] x The x coordinate
  # @param [Integer] y The y coordinate
  # @param [String] opp The opponent's nickname
  # @return [NotImportant] Not important value
  def doHit(opp,x,y)
    @gameFrames[opp].doHit(x.to_i,y.to_i)
  end
  
  # Close the specified {GameFrame game frame}
  # @param [String] opp The opponent's nickname
  # @return [NotImportant] Not important value
  def closeGameFrame(opp)
    @gameFrames[opp].hide
    @gameFrames[opp].destroy
    @gameFrames.delete(opp)
  end
 
  # Send a message about the player lost the game and pop up a window about it.
  # @param [String] opp The opponent's nickname
  # @return [NotImportant] Not important value
  def sendLost(opp)
    @closertimer = nil
    @closertimer = Wx::Timer.new(self,6)
    evt_timer(@closertimer.get_id) do
      @client.send("game#lost##{opp}")
      Wx::message_box("Kedves #{@mainframe.nick} kikaptal az #{opp}-al folytatott csataban!","Vesztettel",Wx::OK)
      closeGameFrame(opp)
    end
    @closertimer.start(100,true)
  end

  # Pop up a window wit a message about the player won the game
  # @param [String] opp The opponent's nickname
  # @return [NotImportant] Not important value
  def win(opp)
    @closertimer = nil
    @closertimer = Wx::Timer.new(self,6)
    evt_timer(@closertimer.get_id) do
      @client.send("game#lost##{opp}")
      Wx::message_box("Kedves #{@mainframe.nick} nyertel az #{opp}-al folytatott csataban!","Nyertel",Wx::OK)
      closeGameFrame(opp)
    end
    @closertimer.start(100,true)
  end
end

end
